@@ -11,7 +11,6 @@
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: calendar.c,v 1.13 2005/09/30 03:29:32 dfs Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -24,6 +23,7 @@ static char const RCSID[] = "$Id: calendar.c,v 1.13 2005/09/30 03:29:32 dfs Exp
 #include "expr.h"
 #include "globals.h"
 #include "err.h"
+#include "md5.h"
 
 /* Data structures used by the calendar */
 typedef struct cal_entry {
@@ -41,7 +41,6 @@ typedef struct cal_entry {
 
 /* Global variables */
 static CalEntry *CalColumn[7];
-static CalEntry *CalPs[7];
 
 static int ColSpaces;
 
@@ -588,13 +587,13 @@ static int DoCalRem(ParsePtr p, int col)
     int r;
     int jul;
     CalEntry *CurCol = CalColumn[col];
-    CalEntry *CurPs = CalPs[col];
     CalEntry *e;
     char *s, *s2;
-    DynamicBuffer buf, obuf;
+    DynamicBuffer buf, obuf, pre_buf;
     Token tok;
 
     DBufInit(&buf);
+    DBufInit(&pre_buf);
 
     /* Parse the trigger date and time */
     if ( (r=ParseRem(p, &trig, &tim)) ) return r;
@@ -638,22 +637,56 @@ static int DoCalRem(ParsePtr p, int col)
 	strcpy(trig.passthru, "PSFile");
 	trig.typ = PASSTHRU_TYPE;
     }
-    if (!PsCal && trig.typ == PASSTHRU_TYPE) return OK;
-
-    /* Remove any "at" times from PS or PSFILE reminders */
-    if (trig.typ == PASSTHRU_TYPE && strcmp(trig.passthru, "COLOR")) {
-	tim.ttime = NO_TIME;
-	tim.duration = NO_TIME;
+    if (trig.typ == PASSTHRU_TYPE) {
+      if (!PsCal && strcmp(trig.passthru, "COLOR")) return OK;
+      if (!strcmp(trig.passthru, "COLOR")) {
+	  /* Strip off the three color numbers */
+	  DBufFree(&buf);
+	  r=ParseToken(p, &buf);
+	  DBufPuts(&pre_buf, DBufValue(&buf));
+	  DBufPutc(&pre_buf, ' ');
+	  DBufFree(&buf);
+	  if (r) return r;
+	  r=ParseToken(p, &buf);
+	  DBufPuts(&pre_buf, DBufValue(&buf));
+	  DBufPutc(&pre_buf, ' ');
+	  DBufFree(&buf);
+	  if (r) return r;
+	  r=ParseToken(p, &buf);
+	  DBufPuts(&pre_buf, DBufValue(&buf));
+	  DBufPutc(&pre_buf, ' ');
+	  DBufFree(&buf);
+	  if (r) return r;
+	  if (!PsCal && !DoSimpleCalendar) {
+	      DBufFree(&pre_buf);
+	  }
+      }
     }
 
     /* If trigger date == today, add it to the current entry */
     DBufInit(&obuf);
-    if (jul == JulianToday) {
+    if ((jul == JulianToday) ||
+	(DoSimpleCalendar &&
+	 DoSimpleCalDelta &&
+	 ShouldTriggerReminder(&trig, &tim, jul))) {
 	NumTriggered++;
+
 	if (DoSimpleCalendar || tim.ttime != NO_TIME) {
-	    if (DBufPuts(&obuf, SimpleTime(tim.ttime)) != OK) {
-		DBufFree(&obuf);
-		return E_NO_MEM;
+	    /* Suppress time if it's not today or if it's a non-COLOR special */
+	    if (jul != JulianToday ||
+		(trig.typ == PASSTHRU_TYPE &&
+		 strcmp(trig.passthru, "COLOR"))) {
+		if (DBufPuts(&obuf, SimpleTime(NO_TIME)) != OK) {
+		    DBufFree(&obuf);
+		    DBufFree(&pre_buf);
+		    return E_NO_MEM;
+		}
+	    } else {
+		if (DBufPuts(&obuf, CalendarTime(tim.ttime, tim.duration)) != OK) {
+		    DBufFree(&obuf);
+		    DBufFree(&pre_buf);
+		    return E_NO_MEM;
+		}
 	    }
 	}
 	if (trig.typ != PASSTHRU_TYPE &&
@@ -667,6 +700,7 @@ static int DoCalRem(ParsePtr p, int col)
 		    if (DBufPuts(&obuf, v.v.str) != OK) {
 			DestroyValue(v);
 			DBufFree(&obuf);
+			DBufFree(&pre_buf);
 			return E_NO_MEM;
 		    }
 		}
@@ -674,12 +708,22 @@ static int DoCalRem(ParsePtr p, int col)
 	    }
 	}
 	oldLen = DBufLen(&obuf);
-	if ( (r=DoSubst(p, &obuf, &trig, &tim, jul, CAL_MODE)) ) {
+
+	/* In -sa mode, run in ADVANCE mode if we're triggering
+	 * before the actual date */
+	if (jul != JulianToday) {
+	    r = DoSubst(p, &obuf, &trig, &tim, jul, ADVANCE_MODE);
+	} else {
+	    r = DoSubst(p, &obuf, &trig, &tim, jul, CAL_MODE);
+	}
+	if (r) {
+	    DBufFree(&pre_buf);
 	    DBufFree(&obuf);
 	    return r;
 	}
 	if (DBufLen(&obuf) <= oldLen) {
 	    DBufFree(&obuf);
+	    DBufFree(&pre_buf);
 	    return OK;
 	}
 	if (trig.typ != PASSTHRU_TYPE &&
@@ -693,6 +737,7 @@ static int DoCalRem(ParsePtr p, int col)
 		    if (DBufPuts(&obuf, v.v.str) != OK) {
 			DestroyValue(v);
 			DBufFree(&obuf);
+			DBufFree(&pre_buf);
 			return E_NO_MEM;
 		    }
 		}
@@ -701,20 +746,28 @@ static int DoCalRem(ParsePtr p, int col)
 	}
 	s = DBufValue(&obuf);
 	if (!DoSimpleCalendar) while (isspace(*s)) s++;
+	DBufPuts(&pre_buf, s);
+	s = DBufValue(&pre_buf);
 	e = NEW(CalEntry);
 	if (!e) {
 	    DBufFree(&obuf);
+	    DBufFree(&pre_buf);
 	    return E_NO_MEM;
 	}
 	e->text = StrDup(s);
 	DBufFree(&obuf);
+	DBufFree(&pre_buf);
 	if (!e->text) {
 	    free(e);
 	    return E_NO_MEM;
 	}
 	StrnCpy(e->tag, trig.tag, TAG_LEN);
 	if (!e->tag[0]) {
-	    strcpy(e->tag, "*");
+	    if (SynthesizeTags) {
+		SynthesizeTag(e->tag);
+	    } else {
+		strcpy(e->tag, "*");
+	    }
 	}
 	e->duration = tim.duration;
 	e->priority = trig.priority;
@@ -725,28 +778,20 @@ static int DoCalRem(ParsePtr p, int col)
 	}
 	e->lineno = LineNo;
 
-	/* Ugly hack... a SPECIAL COLOR is not treated as "passthru"
-	   to preserve ordering and make it behave like a MSG-type reminder */
-
-	if (trig.typ == PASSTHRU_TYPE && strcmp(trig.passthru, "COLOR")) {
+	if (trig.typ == PASSTHRU_TYPE) {
 	    StrnCpy(e->passthru, trig.passthru, PASSTHRU_LEN);
-	    e->pos = e->passthru;
-	    e->time = NO_TIME;
-	    e->next = CurPs;
-	    CalPs[col] = e;
-	    SortCol(&CalPs[col]);
 	} else {
-	    if (trig.typ == PASSTHRU_TYPE) {
-		StrnCpy(e->passthru, trig.passthru, PASSTHRU_LEN);
-	    } else {
-		e->passthru[0] = 0;
-	    }
-	    e->pos = e->text;
-	    e->time = tim.ttime;
-	    e->next = CurCol;
-	    CalColumn[col] = e;
-	    SortCol(&CalColumn[col]);
+	    e->passthru[0] = 0;
 	}
+	e->pos = e->text;
+	if (jul == JulianToday) {
+	    e->time = tim.ttime;
+	} else {
+	    e->time = NO_TIME;
+	}
+	e->next = CurCol;
+	CalColumn[col] = e;
+	SortCol(&CalColumn[col]);
     }
     return OK;
 }
@@ -760,37 +805,11 @@ static int DoCalRem(ParsePtr p, int col)
 /***************************************************************/
 static void WriteSimpleEntries(int col, int jul)
 {
-    CalEntry *e = CalPs[col];
+    CalEntry *e = CalColumn[col];
     CalEntry *n;
     int y, m, d;
 
-/* Do all the PASSTHRU entries first, if any */
     FromJulian(jul, &y, &m, &d);
-    while(e) {
-	if (DoPrefixLineNo) printf("# fileinfo %d %s\n", e->lineno, e->filename);
-	printf("%04d/%02d/%02d ", y, m+1, d);
-	printf("%s ", e->passthru);
-	printf("%s ", e->tag);
-	if (e->duration != NO_TIME) {
-	    printf("%d ", e->duration);
-	} else {
-	    printf("* ");
-	}
-	if (e->time != NO_TIME) {
-	    printf("%d ", e->time);
-	} else {
-	    printf("* ");
-	}
-	printf("%s\n", e->text);
-	free(e->text);
-	free(e->filename);
-	n = e->next;
-	free(e);
-	e = n;
-    }
-    CalPs[col] = NULL;
-
-    e = CalColumn[col];
     while(e) {
 	if (DoPrefixLineNo) printf("# fileinfo %d %s\n", e->lineno, e->filename);
 	printf("%04d/%02d/%02d", y, m+1, d);
@@ -859,6 +878,89 @@ static void WriteCalDays(void)
     PutChar('\n');
 }
 
+/***************************************************************/
+/*                                                             */
+/*  CalendarTime                                               */
+/*                                                             */
+/*  Format the time according to simple time format.           */
+/*  Answer is returned in a static buffer.                     */
+/*  A trailing space is always added.                          */
+/*  This takes into account duration                           */
+/*                                                             */
+/***************************************************************/
+char *
+CalendarTime(int tim, int duration)
+{
+    static char buf[128];
+    int h, min, hh;
+    int h2, min2, hh2, newtim, days;
+    char *ampm1;
+    char *ampm2;
+    char daybuf[64];
+
+    buf[0] = 0;
+
+    if (duration == NO_TIME) {
+	/* No duration... just call into SimpleTime */
+	return SimpleTime(tim);
+    }
+    if (tim == NO_TIME) {
+	/* No time... nothing to return */
+	return buf;
+    }
+    h = tim/60;
+    min = tim % 60;
+    if (h == 0)      hh=12;
+    else if (h > 12) hh=h-12;
+    else             hh = h;
+
+    newtim = tim + duration;
+
+    /* How many days in duration? */
+    days = newtim / MINUTES_PER_DAY;
+    newtim = newtim % MINUTES_PER_DAY;
+    h2 = newtim/60;
+    min2 = newtim % 60;
+    if (h2 == 0)      hh2=12;
+    else if (h2 > 12) hh2=h2-12;
+    else              hh2 = h2;
+
+    if (days) {
+	sprintf(daybuf, "+%d", days);
+    } else {
+	daybuf[0] = 0;
+    }
+
+    if (h >= 12) {
+	ampm1 = L_PM;
+    } else {
+	ampm1 = L_AM;
+    }
+    if (h2 >= 12) {
+	ampm2 = L_PM;
+    } else {
+	ampm2 = L_AM;
+    }
+    if (!days) {
+	if (!strcmp(ampm1, ampm2)) {
+	    ampm1 = "";
+	}
+    }
+
+    switch(ScFormat) {
+    case SC_AMPM:
+	sprintf(buf, "%d%c%02d%s-%d%c%02d%s%s ",
+		hh, TimeSep, min, ampm1, hh2, TimeSep, min2, ampm2, daybuf);
+	break;
+
+    case SC_MIL:
+	sprintf(buf, "%02d%c%02d-%02d%c%02d%s ",
+		h, TimeSep, min, h2, TimeSep, min2, daybuf);
+	break;
+    }
+    return buf;
+}
+
 /***************************************************************/
 /*                                                             */
 /*  SimpleTime                                                 */
@@ -884,7 +986,7 @@ char *SimpleTime(int tim)
 	    if (h == 0) hh=12;
 	    else if (h > 12) hh=h-12;
 	    else hh=h;
-	    sprintf(buf, "%2d%c%02d%s ", hh, TIMESEP, min, (h>=12) ? L_PM : L_AM);
+	    sprintf(buf, "%d%c%02d%s ", hh, TimeSep, min, (h>=12) ? L_PM : L_AM);
 	}
 	break;
 
@@ -892,7 +994,7 @@ char *SimpleTime(int tim)
 	if (tim != NO_TIME) {
 	    h = tim / 60;
 	    min = tim % 60;
-	    sprintf(buf, "%02d%c%02d ", h, TIMESEP, min);
+	    sprintf(buf, "%02d%c%02d ", h, TimeSep, min);
 	}
 	break;
     }
@@ -935,3 +1037,22 @@ static void SortCol(CalEntry **col)
 	}
     }
 }
+
+void SynthesizeTag(char *out)
+{
+    struct MD5Context ctx;
+    unsigned char buf[16];
+    MD5Init(&ctx);
+    MD5Update(&ctx, (unsigned char *) CurLine, strlen(CurLine));
+    MD5Final(buf, &ctx);
+    sprintf(out, "__syn__%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
+	    (unsigned int) buf[0], (unsigned int) buf[1],
+	    (unsigned int) buf[2], (unsigned int) buf[3],
+	    (unsigned int) buf[4], (unsigned int) buf[5],
+	    (unsigned int) buf[6], (unsigned int) buf[7],
+	    (unsigned int) buf[8], (unsigned int) buf[9],
+	    (unsigned int) buf[10], (unsigned int) buf[11],
+	    (unsigned int) buf[12], (unsigned int) buf[13],
+	    (unsigned int) buf[14], (unsigned int) buf[15]);
+}
+
