@@ -13,8 +13,6 @@
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: dorem.c,v 1.14 2005/11/20 01:26:59 dfs Exp $";
-
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -33,7 +31,7 @@ static char const DontEscapeMe[] =
 
 static int ParseTimeTrig (ParsePtr s, TimeTrig *tim);
 static int ParseLocalOmit (ParsePtr s, Trigger *t);
-static int ParseScanFrom (ParsePtr s, Trigger *t);
+static int ParseScanFrom (ParsePtr s, Trigger *t, int type);
 static int ParsePriority (ParsePtr s, Trigger *t);
 static int ParseUntil (ParsePtr s, Trigger *t);
 static int ShouldTriggerBasedOnWarn (Trigger *t, int jul);
@@ -64,6 +62,7 @@ int DoRem(ParsePtr p)
     if (trig.typ == SAT_TYPE) {
 	r=DoSatRemind(&trig, &tim, p);
 	if (r) return r;
+	if (!LastTrigValid) return OK;
 	r=ParseToken(p, &buf);
 	if (r) return r;
 	FindToken(DBufValue(&buf), &tok);
@@ -147,10 +146,12 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     trig->sched[0] = 0;
     trig->warn[0] = 0;
     trig->tag[0] = 0;
+    trig->passthru[0] = 0;
     tim->ttime = NO_TIME;
     tim->delta = NO_DELTA;
     tim->rep   = NO_REP;
     tim->duration = NO_TIME;
+    LastTriggerTime = NO_TIME;
 
     while(1) {
 	/* Read space-delimited string */
@@ -192,7 +193,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
 
 	case T_Scanfrom:
 	    DBufFree(&buf);
-	    r=ParseScanFrom(s, trig);
+	    r=ParseScanFrom(s, trig, tok.val);
 	    if (r) return r;
 	    break;
 
@@ -462,10 +463,10 @@ static int ParseUntil(ParsePtr s, Trigger *t)
 
 /***************************************************************/
 /*                                                             */
-/*  ParseScanFrom - parse the SCANFROM portion of a reminder   */
+/*  ParseScanFrom - parse the FROM/SCANFROM portion            */
 /*                                                             */
 /***************************************************************/
-static int ParseScanFrom(ParsePtr s, Trigger *t)
+static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 {
     int y = NO_YR,
 	m = NO_MON,
@@ -474,7 +475,14 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
     Token tok;
     int r;
     DynamicBuffer buf;
+    char const *word;
+
     DBufInit(&buf);
+    if (type == SCANFROM_TYPE) {
+	word = "SCANFROM";
+    } else {
+	word = "FROM";
+    }
 
     if (t->scanfrom != NO_DATE) return E_SCAN_TWICE;
 
@@ -486,7 +494,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
 	case T_Year:
 	    DBufFree(&buf);
 	    if (y != NO_YR) {
-		Eprint("SCANFROM: %s", ErrMsg[E_YR_TWICE]);
+		Eprint("%s: %s", word, ErrMsg[E_YR_TWICE]);
 		return E_YR_TWICE;
 	    }
 	    y = tok.val;
@@ -495,7 +503,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
 	case T_Month:
 	    DBufFree(&buf);
 	    if (m != NO_MON) {
-		Eprint("SCANFROM: %s", ErrMsg[E_MON_TWICE]);
+		Eprint("%s: %s", word, ErrMsg[E_MON_TWICE]);
 		return E_MON_TWICE;
 	    }
 	    m = tok.val;
@@ -504,7 +512,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
 	case T_Day:
 	    DBufFree(&buf);
 	    if (d != NO_DAY) {
-		Eprint("SCANFROM: %s", ErrMsg[E_DAY_TWICE]);
+		Eprint("%s: %s", word, ErrMsg[E_DAY_TWICE]);
 		return E_DAY_TWICE;
 	    }
 	    d = tok.val;
@@ -512,7 +520,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
 
 	default:
 	    if (y == NO_YR || m == NO_MON || d == NO_DAY) {
-		Eprint("SCANFROM: %s", ErrMsg[E_INCOMPLETE]);
+		Eprint("%s: %s", word, ErrMsg[E_INCOMPLETE]);
 		DBufFree(&buf);
 		return E_INCOMPLETE;
 	    }
@@ -521,6 +529,12 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
 		return E_BAD_DATE;
 	    }
 	    t->scanfrom = Julian(y, m, d);
+	    if (type == FROM_TYPE) {
+		if (t->scanfrom < JulianToday) {
+		    t->scanfrom = JulianToday;
+		}
+	    }
+
 	    PushToken(DBufValue(&buf), s);
 	    DBufFree(&buf);
 	    return OK;
@@ -534,23 +548,46 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
 /*  Trigger the reminder if it's a RUN or MSG type.            */
 /*                                                             */
 /***************************************************************/
-    int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
+int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 {
     int r, y, m, d;
     char PrioExpr[25];
     char tmpBuf[64];
     DynamicBuffer buf, calRow;
+    DynamicBuffer pre_buf;
     char *s;
     Value v;
 
     DBufInit(&buf);
     DBufInit(&calRow);
+    DBufInit(&pre_buf);
     if (t->typ == RUN_TYPE && RunDisabled) return E_RUN_DISABLED;
-    if (t->typ == PASSTHRU_TYPE ||
+    if ((t->typ == PASSTHRU_TYPE && strcmp(t->passthru, "COLOR")) ||
 	t->typ == CAL_TYPE ||
 	t->typ == PS_TYPE ||
 	t->typ == PSF_TYPE)
 	return OK;
+
+    /* Handle COLOR types */
+    if (t->typ == PASSTHRU_TYPE && !strcmp(t->passthru, "COLOR")) {
+	/* Strip off three tokens */
+	r = ParseToken(p, &buf);
+	DBufPuts(&pre_buf, DBufValue(&buf));
+	DBufPutc(&pre_buf, ' ');
+	DBufFree(&buf);
+	if (r) return r;
+	r = ParseToken(p, &buf);
+	DBufPuts(&pre_buf, DBufValue(&buf));
+	DBufPutc(&pre_buf, ' ');
+	DBufFree(&buf);
+	if (r) return r;
+	r = ParseToken(p, &buf);
+	DBufPuts(&pre_buf, DBufValue(&buf));
+	DBufPutc(&pre_buf, ' ');
+	DBufFree(&buf);
+	if (r) return r;
+	t->typ = MSG_TYPE;
+    }
 /* If it's a MSG-type reminder, and no -k option was used, issue the banner. */
     if ((t->typ == MSG_TYPE || t->typ == MSF_TYPE) 
 	&& !NumTriggered && !NextMode && !MsgCommand) {
@@ -568,12 +605,14 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
 	if ( (r=DoSubst(p, &buf, t, tim, jul, CAL_MODE)) ) return r;
 	if (!DBufLen(&buf)) {
 	    DBufFree(&buf);
+	    DBufFree(&pre_buf);
 	    return OK;
 	}
 	FromJulian(jul, &y, &m, &d);
- 	sprintf(tmpBuf, "%04d%c%02d%c%02d ", y, DATESEP, m+1, DATESEP, d);
+ 	sprintf(tmpBuf, "%04d/%02d/%02d ", y, m+1, d);
  	if (DBufPuts(&calRow, tmpBuf) != OK) {
  	    DBufFree(&calRow);
+	    DBufFree(&pre_buf);
  	    return E_NO_MEM;
  	}
  	/* If DoSimpleCalendar==1, output *all* simple calendar fields */
@@ -581,6 +620,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
  	    /* ignore passthru field when in NextMode */
  	    if (DBufPuts(&calRow, "* ") != OK) {
  		DBufFree(&calRow);
+		DBufFree(&pre_buf);
  		return E_NO_MEM;
  	    }
  	    if (t->tag[0]) {
@@ -590,6 +630,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
  	    }
  	    if (DBufPuts(&calRow, tmpBuf) != OK) {
  		DBufFree(&calRow);
+		DBufFree(&pre_buf);
  		return E_NO_MEM;
  	    }
  	    if (tim->duration != NO_TIME) {
@@ -599,6 +640,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
  	    }
  	    if (DBufPuts(&calRow, tmpBuf) != OK) {
  		DBufFree(&calRow);
+		DBufFree(&pre_buf);
  		return E_NO_MEM;
  	    }
  	    if (tim->ttime != NO_TIME) {
@@ -608,16 +650,19 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
  	    }
  	    if (DBufPuts(&calRow, tmpBuf) != OK) {
  		DBufFree(&calRow);
+		DBufFree(&pre_buf);
  		return E_NO_MEM;
  	    }
  	}
  	if (DBufPuts(&calRow, SimpleTime(tim->ttime)) != OK) {
  	    DBufFree(&calRow);
+	    DBufFree(&pre_buf);
  	    return E_NO_MEM;
  	}
 
- 	printf("%s%s\n", DBufValue(&calRow), DBufValue(&buf));
+ 	printf("%s%s%s\n", DBufValue(&calRow), DBufValue(&pre_buf), DBufValue(&buf));
 	DBufFree(&buf);
+	DBufFree(&pre_buf);
 	DBufFree(&calRow);
 	return OK;
     }
@@ -675,6 +720,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t)
    reminder now. */
     switch(t->typ) {
     case MSG_TYPE:
+    case PASSTHRU_TYPE:
 	if (MsgCommand) {
 	    DoMsgCommand(MsgCommand, DBufValue(&buf));
 	} else {
@@ -776,6 +822,9 @@ int DoSatRemind(Trigger *trig, TimeTrig *tim, ParsePtr p)
 	if (r) {
 	    if (r == E_CANT_TRIG) return OK; else return r;
 	}
+	if (jul == -1) {
+	    return OK;
+	}
 	s = p->pos;
 	r = EvaluateExpr(p, &v);
 	t = p->pos;
@@ -865,7 +914,7 @@ int DoMsgCommand(char *cmd, char *msg)
     msg = DBufValue(&buf);
 
     /* Do "%s" substitution */
-    l = strlen(cmd)-1;
+    l = strlen(cmd);
     for (i=0; i<l; i++) {
 	if (cmd[i] == '%' && cmd[i+1] == 's') {
 	    ++i;
@@ -880,11 +929,6 @@ int DoMsgCommand(char *cmd, char *msg)
 	    }
 	}
     }
-    if (l >= 0 && DBufPutc(&execBuffer, cmd[l]) != OK) {
-	r = E_NO_MEM;
-	goto finished;
-    }
-
     r = OK;
 
     system(DBufValue(&execBuffer));
