@@ -12,8 +12,6 @@
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: dosubst.c,v 1.11 2005/09/30 03:29:32 dfs Exp $";
-
 #define L_IN_DOSUBST
 #include <stdio.h>
 #include <string.h>
@@ -44,6 +42,7 @@ static char TOMORROW[] = L_TOMORROW;
 /*  Process the % escapes in the reminder.  If                 */
 /*  mode==NORMAL_MODE, ignore the %" sequence.  If             */
 /*  mode==CAL_MODE, process the %" sequence.                   */
+/*  If mode==ADVANCE_MODE, ignore %" but don't add newline     */
 /*                                                             */
 /***************************************************************/
 int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul, int mode)
@@ -131,7 +130,8 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	}
 	if (c == '\n') continue;
 	if (!c) {
-	    if (mode != CAL_MODE && t->typ != RUN_TYPE && !MsgCommand) {
+	    if (mode != CAL_MODE && mode != ADVANCE_MODE &&
+		t->typ != RUN_TYPE && !MsgCommand) {
 		if (DBufPutc(dbuf, '\n') != OK) return E_NO_MEM;
 	    }
 	    break;
@@ -242,8 +242,8 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_E_OVER
 	    L_E_OVER
 #else
-	    sprintf(s, "%s %02d%c%02d%c%04d", L_ON, d, DATESEP,
-		    m+1, DATESEP, y);
+	    sprintf(s, "%s %02d%c%02d%c%04d", L_ON, d, DateSep,
+		    m+1, DateSep, y);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -252,7 +252,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_F_OVER
 	    L_F_OVER
 #else
-	    sprintf(s, "%s %02d%c%02d%c%04d", L_ON, m+1, DATESEP, d, DATESEP, y);
+	    sprintf(s, "%s %02d%c%02d%c%04d", L_ON, m+1, DateSep, d, DateSep, y);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -270,7 +270,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_H_OVER
 	    L_H_OVER
 #else
-	    sprintf(s, "%s %02d%c%02d", L_ON, d, DATESEP, m+1);
+	    sprintf(s, "%s %02d%c%02d", L_ON, d, DateSep, m+1);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -279,7 +279,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_I_OVER
 	    L_I_OVER
 #else
-	    sprintf(s, "%s %02d%c%02d", L_ON, m+1, DATESEP, d);
+	    sprintf(s, "%s %02d%c%02d", L_ON, m+1, DateSep, d);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -308,7 +308,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_L_OVER
 	    L_L_OVER
 #else
-	    sprintf(s, "%s %04d%c%02d%c%02d", L_ON, y, DATESEP, m+1, DATESEP, d);
+	    sprintf(s, "%s %04d%c%02d%c%02d", L_ON, y, DateSep, m+1, DateSep, d);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -463,7 +463,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_2_OVER
 	    L_2_OVER
 #else
-	    sprintf(s, "%s %d%c%02d%s", L_AT, hh, TIMESEP, min, pm);
+	    sprintf(s, "%s %d%c%02d%s", L_AT, hh, TimeSep, min, pm);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -473,7 +473,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	    L_3_OVER
 #else
 
-	    sprintf(s, "%s %02d%c%02d", L_AT, h, TIMESEP, min);
+	    sprintf(s, "%s %02d%c%02d", L_AT, h, TimeSep, min);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -554,7 +554,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_AT_OVER
 	    L_AT_OVER
 #else
-	    sprintf(s, "%d%c%02d%s", chh, TIMESEP, cmin, cpm);
+	    sprintf(s, "%d%c%02d%s", chh, TimeSep, cmin, cpm);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -563,13 +563,13 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_HASH_OVER
 	    L_HASH_OVER
 #else
-	    sprintf(s, "%02d%c%02d", ch, TIMESEP, cmin);
+	    sprintf(s, "%02d%c%02d", ch, TimeSep, cmin);
 #endif
 	    SHIP_OUT(s);
 	    break;
 
 	case '_':
-	    if (mode != CAL_MODE && !MsgCommand)
+	    if (mode != CAL_MODE && mode != ADVANCE_MODE && !MsgCommand)
 		sprintf(s, "%s", NL);
 	    else
 		sprintf(s, " ");
@@ -602,7 +602,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 /* If there are NO quotes, then:  If CAL_MODE && RUN_TYPE, we don't want the
    reminder in the calendar.  Zero the output buffer and quit. */
     if (!has_quote) {
-	if (mode == CAL_MODE && t->typ == RUN_TYPE) {
+	if ((mode == ADVANCE_MODE || mode == CAL_MODE) && t->typ == RUN_TYPE) {
 	    *DBufValue(dbuf) = 0;
 	    dbuf->len = 0;
 	}
@@ -614,7 +614,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 
     ss = DBufValue(dbuf) + origLen;
     os = ss;
-    if (mode == NORMAL_MODE) {
+    if (mode == NORMAL_MODE || mode == ADVANCE_MODE) {
 	while (*ss) {
 	    if (*ss != QUOTE_MARKER) *os++ = *ss;
 	    ss++;
