@@ -11,9 +11,6 @@
 /*                                                             */
 /***************************************************************/
 
-static char const RCSID[] =
-"$Id: dynbuf.c,v 1.6 2005/09/30 03:29:32 dfs Exp $";
-
 #include "config.h"
 #include "dynbuf.h"
 #include "err.h"
@@ -76,7 +73,7 @@ void DBufInit(DynamicBuffer *dbuf)
 }
 
 /**********************************************************************
-%FUNCTION: DBufPutc
+%FUNCTION: DBufPutcFN
 %ARGUMENTS:
  dbuf -- pointer to a dynamic buffer
  c -- character to append to buffer
@@ -85,7 +82,7 @@ void DBufInit(DynamicBuffer *dbuf)
 %DESCRIPTION:
  Appends a character to the buffer.
 **********************************************************************/
-int DBufPutc(DynamicBuffer *dbuf, char c)
+int DBufPutcFN(DynamicBuffer *dbuf, char c)
 {
     if (dbuf->allocatedLen == dbuf->len+1) {
 	if (DBufMakeRoom(dbuf, dbuf->len+1) != OK) return E_NO_MEM;
