@@ -10,8 +10,6 @@
 /*                                                             */
 /***************************************************************/
 
-/* $Id: dynbuf.h,v 1.4 2005/09/30 03:29:32 dfs Exp $ */
-
 #ifndef DYNBUF_H
 #define DYNBUF_H
 
@@ -26,7 +24,7 @@ typedef struct {
 } DynamicBuffer;
 
 void DBufInit(DynamicBuffer *dbuf);
-int DBufPutc(DynamicBuffer *dbuf, char c);
+int DBufPutcFN(DynamicBuffer *dbuf, char c);
 int DBufPuts(DynamicBuffer *dbuf, char *str);
 void DBufFree(DynamicBuffer *dbuf);
 int DBufGets(DynamicBuffer *dbuf, FILE *fp);
@@ -34,4 +32,6 @@ int DBufGets(DynamicBuffer *dbuf, FILE *fp);
 #define DBufValue(bufPtr) ((bufPtr)->buffer)
 #define DBufLen(bufPtr) ((bufPtr)->len)
 
+#define DBufPutc(dbuf, c) ( (dbuf)->allocatedLen < (dbuf)->len+1 ) ? (dbuf)->buffer[(dbuf)->len++] = c, (dbuf)->buffer[(dbuf)->len] = 0, OK : DBufPutcFN((dbuf), c)
+
 #endif /* DYNBUF_H */
