@@ -11,8 +11,6 @@
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: expr.c,v 1.10 2005/09/30 03:29:32 dfs Exp $";
-
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -44,7 +42,7 @@ static int Multiply(void), Divide(void), Mod(void), Add(void),
     Compare(int);
 
 static int MakeValue (char *s, Value *v, Var *locals);
-static int ParseLiteralDate (char **s, int *jul);
+static int ParseLiteralDate (char **s, int *jul, int *tim);
 
 /* Binary operators - all left-associative */
 
@@ -240,7 +238,7 @@ static int ParseExprToken(DynamicBuffer *buf, char **in)
     }
 
     /* Parse a constant, variable name or function */
-    while (ISID(**in) || **in == ':' || **in == '.' || **in == TIMESEP) {
+    while (ISID(**in) || **in == ':' || **in == '.' || **in == TimeSep) {
 	if (DBufPutc(buf, **in) != OK) {
 	    DBufFree(buf);
 	    return E_NO_MEM;
@@ -368,8 +366,8 @@ int Evaluate(char **s, Var *locals)
 		    r = CallUserFunc(ufname, args);
 		    free(ufname);
 		}
-		if (r) return r;
 		DBufFree(&ExprBuf);
+		if (r) return r;
 	    }
 	} else { /* Unary operator */
 	    f = FindFunc(DBufValue(&ExprBuf), UnOp, NUM_UN_OPS);
@@ -468,10 +466,15 @@ static int MakeValue(char *s, Value *v, Var *locals)
 	return OK;
     } else if (*s == '\'') { /* It's a literal date */
 	s++;
-	if ((r=ParseLiteralDate(&s, &h))) return r;
+	if ((r=ParseLiteralDate(&s, &h, &m))) return r;
 	if (*s != '\'') return E_BAD_DATE;
-	v->type = DATE_TYPE;
-	v->v.val = h;
+	if (m == NO_TIME) {
+	    v->type = DATE_TYPE;
+	    v->v.val = h;
+	} else {
+	    v->type = DATETIME_TYPE;
+	    v->v.val = (h * MINUTES_PER_DAY) + m;
+	}
 	return OK;
     } else if (isdigit(*s)) { /* It's a number - use len to hold it.*/
 	len = 0;
@@ -479,7 +482,7 @@ static int MakeValue(char *s, Value *v, Var *locals)
 	    len *= 10;
 	    len += (*s++ - '0');
 	}
-	if (*s == ':' || *s == '.' || *s == TIMESEP) { /* Must be a literal time */
+	if (*s == ':' || *s == '.' || *s == TimeSep) { /* Must be a literal time */
 	    s++;
 	    if (!isdigit(*s)) return E_BAD_TIME;
 	    h = len;
@@ -490,7 +493,7 @@ static int MakeValue(char *s, Value *v, Var *locals)
 		s++;
 	    }
 	    if (*s || h>23 || m>59) return E_BAD_TIME;
-	    v->type = TIM_TYPE;
+	    v->type = TIME_TYPE;
 	    v->v.val = h*60 + m;
 	    return OK;
 	}
@@ -529,22 +532,52 @@ static int MakeValue(char *s, Value *v, Var *locals)
 /***************************************************************/
 int DoCoerce(char type, Value *v)
 {
-    int h, d, m, y, i;
+    int h, d, m, y, i, k;
     char *s;
 
     /* Do nothing if value is already the right type */
     if (type == v->type) return OK;
 
     switch(type) {
+    case DATETIME_TYPE:
+	switch(v->type) {
+	case INT_TYPE:
+	    v->type = DATETIME_TYPE;
+	    return OK;
+	case DATE_TYPE:
+	    v->type = DATETIME_TYPE;
+	    v->v.val *= MINUTES_PER_DAY;
+	    return OK;
+	case STR_TYPE:
+	    s = v->v.str;
+	    if (ParseLiteralDate(&s, &i, &m)) return E_CANT_COERCE;
+	    if (*s) return E_CANT_COERCE;
+	    v->type = DATETIME_TYPE;
+	    free(v->v.str);
+	    if (m == NO_TIME) m = 0;
+	    v->v.val = i * MINUTES_PER_DAY + m;
+	    return OK;
+	default:
+	    return E_CANT_COERCE;
+	}
     case STR_TYPE:
 	switch(v->type) {
 	case INT_TYPE: sprintf(CoerceBuf, "%d", v->v.val); break;
-	case TIM_TYPE: sprintf(CoerceBuf, "%02d%c%02d", v->v.val / 60,
-			       TIMESEP, v->v.val % 60);
+	case TIME_TYPE: sprintf(CoerceBuf, "%02d%c%02d", v->v.val / 60,
+			       TimeSep, v->v.val % 60);
 	break;
 	case DATE_TYPE: FromJulian(v->v.val, &y, &m, &d);
 	    sprintf(CoerceBuf, "%04d%c%02d%c%02d",
-		    y, DATESEP, m+1, DATESEP, d);
+		    y, DateSep, m+1, DateSep, d);
+	    break;
+	case DATETIME_TYPE:
+	    i = v->v.val / MINUTES_PER_DAY;
+	    FromJulian(i, &y, &m, &d);
+	    k = v->v.val % MINUTES_PER_DAY;
+	    h = k / 60;
+	    i = k % 60;
+	    sprintf(CoerceBuf, "%04d%c%02d%c%02d@%02d%c%02d",
+		    y, DateSep, m+1, DateSep, d, h, TimeSep, i);
 	    break;
 	default: return E_CANT_COERCE;
 	}
@@ -581,7 +614,8 @@ int DoCoerce(char type, Value *v)
 	    return OK;
 
 	case DATE_TYPE:
-	case TIM_TYPE:
+	case TIME_TYPE:
+	case DATETIME_TYPE:
 	    v->type = INT_TYPE;
 	    return OK;
 
@@ -598,22 +632,28 @@ int DoCoerce(char type, Value *v)
 
 	case STR_TYPE:
 	    s = v->v.str;
-	    if (ParseLiteralDate(&s, &i)) return E_CANT_COERCE;
+	    if (ParseLiteralDate(&s, &i, &m)) return E_CANT_COERCE;
 	    if (*s) return E_CANT_COERCE;
 	    v->type = DATE_TYPE;
 	    free(v->v.str);
 	    v->v.val = i;
 	    return OK;
 
+	case DATETIME_TYPE:
+	    v->type = DATE_TYPE;
+	    v->v.val /= MINUTES_PER_DAY;
+	    return OK;
+
 	default: return E_CANT_COERCE;
 	}
 
-    case TIM_TYPE:
+    case TIME_TYPE:
 	switch(v->type) {
 	case INT_TYPE:
-	    v->type = TIM_TYPE;
-	    v->v.val %= 1440;
-	    if (v->v.val < 0) v->v.val += 1440;
+	case DATETIME_TYPE:
+	    v->type = TIME_TYPE;
+	    v->v.val %= MINUTES_PER_DAY;
+	    if (v->v.val < 0) v->v.val += MINUTES_PER_DAY;
 	    return OK;
 
 	case STR_TYPE:
@@ -625,7 +665,7 @@ int DoCoerce(char type, Value *v)
 		h *= 10;
 		h += *s++ - '0';
 	    }
-	    if (*s != ':' && *s != '.' && *s != TIMESEP)
+	    if (*s != ':' && *s != '.' && *s != TimeSep)
 		return E_CANT_COERCE;
 	    s++;
 	    if (!isdigit(*s)) return E_CANT_COERCE;
@@ -634,7 +674,7 @@ int DoCoerce(char type, Value *v)
 		m += *s++ - '0';
 	    }
 	    if (*s || h>23 || m>59) return E_CANT_COERCE;
-	    v->type = TIM_TYPE;
+	    v->type = TIME_TYPE;
 	    free(v->v.str);
 	    v->v.val = h*60+m;
 	    return OK;
@@ -680,12 +720,22 @@ static int Add(void)
 	return OK;
     }
 
-/* If it's a time plus an int, add 'em mod 1440 */
-    if ((v1.type == TIM_TYPE && v2.type == INT_TYPE) ||
-	(v1.type == INT_TYPE && v2.type == TIM_TYPE)) {
-	v1.v.val = (v1.v.val + v2.v.val) % 1440;
-	if (v1.v.val < 0) v1.v.val += 1440;
-	v1.type = TIM_TYPE;
+/* If it's a datetime plus an int, add 'em */
+    if ((v1.type == DATETIME_TYPE && v2.type == INT_TYPE) ||
+	(v1.type == INT_TYPE && v2.type == DATETIME_TYPE)) {
+	v1.v.val += v2.v.val;
+	if (v1.v.val < 0) return E_DATE_OVER;
+	v1.type = DATETIME_TYPE;
+	PushValStack(v1);
+	return OK;
+    }
+
+/* If it's a time plus an int, add 'em mod MINUTES_PER_DAY */
+    if ((v1.type == TIME_TYPE && v2.type == INT_TYPE) ||
+	(v1.type == INT_TYPE && v2.type == TIME_TYPE)) {
+	v1.v.val = (v1.v.val + v2.v.val) % MINUTES_PER_DAY;
+	if (v1.v.val < 0) v1.v.val += MINUTES_PER_DAY;
+	v1.type = TIME_TYPE;
 	PushValStack(v1);
 	return OK;
     }
@@ -750,16 +800,25 @@ static int Subtract(void)
 	return OK;
     }
 
-    /* If it's a time minus an int, do subtraction mod 1440 */
-    if (v1.type == TIM_TYPE && v2.type == INT_TYPE) {
-	v1.v.val = (v1.v.val - v2.v.val) % 1440;
-	if (v1.v.val < 0) v1.v.val += 1440;
+    /* If it's a datetime minus an int, do subtraction, checking for underflow */
+    if (v1.type == DATETIME_TYPE && v2.type == INT_TYPE) {
+	v1.v.val -= v2.v.val;
+	if (v1.v.val < 0) return E_DATE_OVER;
+	PushValStack(v1);
+	return OK;
+    }
+
+    /* If it's a time minus an int, do subtraction mod MINUTES_PER_DAY */
+    if (v1.type == TIME_TYPE && v2.type == INT_TYPE) {
+	v1.v.val = (v1.v.val - v2.v.val) % MINUTES_PER_DAY;
+	if (v1.v.val < 0) v1.v.val += MINUTES_PER_DAY;
 	PushValStack(v1);
 	return OK;
     }
 
     /* If it's a time minus a time or a date minus a date, do it */
-    if ((v1.type == TIM_TYPE && v2.type == TIM_TYPE) ||
+    if ((v1.type == TIME_TYPE && v2.type == TIME_TYPE) ||
+	(v1.type == DATETIME_TYPE && v2.type == DATETIME_TYPE) ||
 	(v1.type == DATE_TYPE && v2.type == DATE_TYPE)) {
 	v1.v.val -= v2.v.val;
 	v1.type = INT_TYPE;
@@ -1053,11 +1112,16 @@ void PrintValue (Value *v, FILE *fp)
 	if (*s) fprintf(fp, "...");
     }
     else if (v->type == INT_TYPE) fprintf(fp, "%d", v->v.val);
-    else if (v->type == TIM_TYPE) fprintf(fp, "%02d%c%02d", v->v.val / 60,
-					  TIMESEP, v->v.val % 60);
+    else if (v->type == TIME_TYPE) fprintf(fp, "%02d%c%02d", v->v.val / 60,
+					   TimeSep, v->v.val % 60);
     else if (v->type == DATE_TYPE) {
 	FromJulian(v->v.val, &y, &m, &d);
-	fprintf(fp, "%04d%c%02d%c%02d", y, DATESEP, m+1, DATESEP, d);
+	fprintf(fp, "%04d%c%02d%c%02d", y, DateSep, m+1, DateSep, d);
+    }
+    else if (v->type == DATETIME_TYPE) {
+	FromJulian(v->v.val / MINUTES_PER_DAY, &y, &m, &d);
+	fprintf(fp, "%04d%c%02d%c%02d@%02d%c%02d", y, DateSep, m+1, DateSep, d,
+		(v->v.val % MINUTES_PER_DAY) / 60, TimeSep, (v->v.val % MINUTES_PER_DAY) % 60);
     }
     else fprintf(fp, "ERR");
 }
@@ -1086,21 +1150,25 @@ int CopyValue(Value *dest, const Value *src)
 /*                                                             */
 /*  ParseLiteralDate                                           */
 /*                                                             */
-/*  Parse a literal date.  Return result in jul, update s.     */
+/*  Parse a literal date or datetime.  Return result in jul    */
+/*  and tim; update s.                                         */
 /*                                                             */
 /***************************************************************/
-static int ParseLiteralDate(char **s, int *jul)
+static int ParseLiteralDate(char **s, int *jul, int *tim)
 {
     int y, m, d;
+    int hour, min;
 
     y=0; m=0; d=0;
+    hour=0; min=0;
 
+    *tim = NO_TIME;
     if (!isdigit(**s)) return E_BAD_DATE;
     while (isdigit(**s)) {
 	y *= 10;
 	y += *(*s)++ - '0';
     }
-    if (**s != '/' && **s != '-' && **s != DATESEP) return E_BAD_DATE;
+    if (**s != '/' && **s != '-' && **s != DateSep) return E_BAD_DATE;
     (*s)++;
     if (!isdigit(**s)) return E_BAD_DATE;
     while (isdigit(**s)) {
@@ -1108,7 +1176,7 @@ static int ParseLiteralDate(char **s, int *jul)
 	m += *(*s)++ - '0';
     }
     m--;
-    if (**s != '/' && **s != '-' && **s != DATESEP) return E_BAD_DATE;
+    if (**s != '/' && **s != '-' && **s != DateSep) return E_BAD_DATE;
     (*s)++;
     if (!isdigit(**s)) return E_BAD_DATE;
     while (isdigit(**s)) {
@@ -1119,6 +1187,23 @@ static int ParseLiteralDate(char **s, int *jul)
 
     *jul = Julian(y, m, d);
 
+    /* Do we have a time part as well? */
+    if (**s == ' ' || **s == '@') {
+	(*s)++;
+	while(isdigit(**s)) {
+	    hour *= 10;
+	    hour += *(*s)++ - '0';
+	}
+	if (**s != ':' && **s != '.' && **s != TimeSep) return E_BAD_TIME;
+	(*s)++;
+	while(isdigit(**s)) {
+	    min *= 10;
+	    min += *(*s)++ - '0';
+	}
+	if (hour > 23 || min > 59) return E_BAD_TIME;
+	*tim = hour * 60 + min;
+    }
+
     return OK;
 }
 
