@@ -12,8 +12,6 @@
 /*                                                             */
 /***************************************************************/
 
-/* $Id: globals.h,v 1.9 2005/09/30 03:29:32 dfs Exp $ */
-
 #ifdef MK_GLOBALS
 #undef EXTERN
 #define EXTERN
@@ -24,6 +22,8 @@
 #define INIT(var, val) var
 #endif
 
+#define MINUTES_PER_DAY 1440
+
 #define DaysInYear(y) (((y) % 4) ? 365 : ((!((y) % 100) && ((y) % 400)) ? 365 : 366 ))
 #define IsLeapYear(y) (((y) % 4) ? 0 : ((!((y) % 100) && ((y) % 400)) ? 0 : 1 ))
 #define DaysInMonth(m, y) ((m) != 1 ? MonthDays[m] : 28 + IsLeapYear(y))
@@ -42,6 +42,7 @@ EXTERN  INIT(	int     ShowAllErrors, 0);
 EXTERN  INIT(	int     DebugFlag, 0);
 EXTERN  INIT(   int	DoCalendar, 0);
 EXTERN  INIT(   int     DoSimpleCalendar, 0);
+EXTERN  INIT(   int     DoSimpleCalDelta, 0);
 EXTERN  INIT(   int     DoPrefixLineNo, 0);
 EXTERN	INIT(	int	MondayFirst, 0);
 EXTERN	INIT(	int	Iterations, 1);
@@ -68,8 +69,9 @@ EXTERN  INIT(	int 	DontQueue, 0);
 EXTERN  INIT(   int     NumQueued, 0);
 EXTERN  INIT(   int     DontIssueAts, 0);
 EXTERN  INIT(   int     Daemon, 0);
-
-
+EXTERN  INIT(   char    DateSep, DATESEP);
+EXTERN  INIT(   char    TimeSep, TIMESEP);
+EXTERN  INIT(   int     SynthesizeTags, 0);
 EXTERN  INIT(   int     ScFormat, SC_AMPM);
 EXTERN  INIT(   int     MaxSatIter, 150);
 EXTERN  INIT(	char	*FileName, NULL);
