@@ -8,12 +8,11 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*  Copyright (C) 1992-1998 by David F. Skoll                  */
-/*  Copyright (C) 1999-2005 by Roaring Penguin Software Inc.   */
+/*  Copyright (C) 1999-2007 by Roaring Penguin Software Inc.   */
 /*                                                             */
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: init.c,v 1.16 2005/09/30 03:29:32 dfs Exp $";
 
 #define L_IN_INIT 1
 #include <stdio.h>
@@ -85,6 +84,37 @@ static void InitializeVar(char *str);
 
 static char *BadDate = "Illegal date on command line\n";
 
+static DynamicBuffer default_filename_buf;
+
+/***************************************************************/
+/*                                                             */
+/*  DefaultFilename                                            */
+/*                                                             */
+/*  If we're invoked as "rem" rather than "remind", use a      */
+/*  default filename.  Use $DOTREMINDERS or $HOME/.reminders   */
+/*                                                             */
+/***************************************************************/
+static char *DefaultFilename(void)
+{
+    char *s;
+
+    DBufInit(&default_filename_buf);
+
+    s = getenv("DOTREMINDERS");
+    if (s) {
+	return s;
+    }
+
+    s = getenv("HOME");
+    if (!s) {
+	fprintf(stderr, "HOME environment variable not set.  Unable to determine reminder file.\n");
+	exit(1);
+    }
+    DBufPuts(&default_filename_buf, s);
+    DBufPuts(&default_filename_buf, "/.reminders");
+    return DBufValue(&default_filename_buf);
+}
+
 /***************************************************************/
 /*                                                             */
 /*  InitRemind                                                 */
@@ -98,6 +128,8 @@ void InitRemind(int argc, char *argv[])
     int i;
     int y, m, d, rep;
     Token tok;
+    int InvokedAsRem = 0;
+    char *s;
 
     /* Initialize global dynamic buffers */
     DBufInit(&Banner);
@@ -126,6 +158,19 @@ void InitRemind(int argc, char *argv[])
     JulianToday = RealToday;
     FromJulian(JulianToday, &CurYear, &CurMon, &CurDay);
 
+    /* See if we were invoked as "rem" rather than "remind" */
+    if (argv[0]) {
+	s = strrchr(argv[0], '/');
+	if (!s) {
+	    s = argv[0];
+	} else {
+	    s++;
+	}
+	if (!strcmp(s, "rem")) {
+	    InvokedAsRem = 1;
+	}
+    }
+
     /* Parse the command-line options */
     i = 1;
     while (i < argc) {
@@ -170,6 +215,11 @@ void InitRemind(int argc, char *argv[])
 		IgnoreOnce = 1;
 		break;
 
+	    case 'y':
+	    case 'Y':
+		SynthesizeTags = 1;
+		break;
+
 	    case 't':
 	    case 'T':
 		InfiniteDelta = 1;
@@ -219,11 +269,11 @@ void InitRemind(int argc, char *argv[])
 		if (*arg == '0') {
 		    PARSENUM(Daemon, arg);
 		    if (Daemon == 0) Daemon = -1;
-		    else if (Daemon < 5) Daemon = 5;
+		    else if (Daemon < 1) Daemon = 1;
 		    else if (Daemon > 60) Daemon = 60;
 		} else {
 		    PARSENUM(Daemon, arg);
-		    if (Daemon<5) Daemon=5;
+		    if (Daemon<1) Daemon=1;
 		    else if (Daemon>60) Daemon=60;
 		}
 		break;
@@ -258,6 +308,10 @@ void InitRemind(int argc, char *argv[])
 	    case 's':
 	    case 'S':
 		DoSimpleCalendar = 1;
+		if (*arg == 'a' || *arg == 'A') {
+			DoSimpleCalDelta = 1;
+			arg++;
+		}
 		if (*arg == '+') {
 		    arg++;
 		    PARSENUM(CalWeeks, arg);
@@ -272,6 +326,10 @@ void InitRemind(int argc, char *argv[])
 	    case 'P':
 		DoSimpleCalendar = 1;
 		PsCal = 1;
+		if (*arg == 'a' || *arg == 'A') {
+		    DoSimpleCalDelta = 1;
+		    arg++;
+		}
 		PARSENUM(CalMonths, arg);
 		if (!CalMonths) CalMonths = 1;
 		break;
@@ -348,11 +406,15 @@ void InitRemind(int argc, char *argv[])
     }
 
     /* Get the filename. */
-    if (i >= argc) {
-	Usage();
-	exit(1);
+    if (!InvokedAsRem) {
+	if (i >= argc) {
+	    Usage();
+	    exit(1);
+	}
+	InitialFile = argv[i++];
+    } else {
+	InitialFile = DefaultFilename();
     }
-    InitialFile = argv[i++];
 
     /* Get the date, if any */
     if (i < argc) {
@@ -444,7 +506,7 @@ void InitRemind(int argc, char *argv[])
 void Usage(void)
 {
     fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-1998 David F. Skoll\n", VERSION, L_LANGNAME);
-    fprintf(ErrFp, "Copyright 1999-2005 Roaring Penguin Software Inc.\n");
+    fprintf(ErrFp, "Copyright 1999-2007 Roaring Penguin Software Inc.\n");
 #ifdef BETA
     fprintf(ErrFp, ">>>> BETA VERSION <<<<\n");
 #endif
@@ -455,8 +517,8 @@ void Usage(void)
     fprintf(ErrFp, " -c[n]  Produce a calendar for n (default 1) months\n");
     fprintf(ErrFp, " -c+[n] Produce a calendar for n (default 1) weeks\n");
     fprintf(ErrFp, " -w[n[,p[,s]]]  Specify width, padding and spacing of calendar\n");
-    fprintf(ErrFp, " -s[+][n] Produce `simple calendar' for n (1) months (weeks)\n");
-    fprintf(ErrFp, " -p[n]  Same as -s, but input compatible with rem2ps\n");
+    fprintf(ErrFp, " -s[a][+][n] Produce `simple calendar' for n (1) months (weeks)\n");
+    fprintf(ErrFp, " -p[a][n] Same as -s, but input compatible with rem2ps\n");
     fprintf(ErrFp, " -l     Prefix each simple calendar line with line number and filename comment\n");
     fprintf(ErrFp, " -v     Verbose mode\n");
     fprintf(ErrFp, " -o     Ignore ONCE directives\n");
@@ -474,6 +536,7 @@ void Usage(void)
     fprintf(ErrFp, " -g[ddd] Sort reminders by date, time and priority before issuing\n");
     fprintf(ErrFp, " -ivar=val Initialize var to val and preserve var\n");
     fprintf(ErrFp, " -m     Start calendar with Monday rather than Sunday\n");
+    fprintf(ErrFp, " -y     Synthesize tags for tagless reminders\n");
     exit(1);
 }
 #endif /* L_USAGE_OVERRIDE */
