@@ -16,8 +16,6 @@
 /*                                                             */
 /***************************************************************/
 
-/* $Id: finnish.h,v 1.10 2000/02/18 03:53:49 dfs Exp $ */
-
 /* The very first define in a language support file must be L_LANGNAME: */
 #define L_LANGNAME "Finnish"
 
@@ -166,18 +164,18 @@
 #define L_A_OVER sprintf(s, "%s%s %d. %s%s %d", DayName[jul%7], L_ON, d, \
 			 MonthName[m], L_PARTIT, y);
 #define L_C_OVER sprintf(s, "%s%s", DayName[jul%7], L_ON);
-#define L_E_OVER sprintf(s, "%02d%c%02d%c%04d", d, DATESEP, m+1, DATESEP, \
+#define L_E_OVER sprintf(s, "%02d%c%02d%c%04d", d, DateSep, m+1, DateSep, \
 			 y);
-#define L_F_OVER sprintf(s, "%02d%c%02d%c%04d", m+1, DATESEP, d, DATESEP, y);
+#define L_F_OVER sprintf(s, "%02d%c%02d%c%04d", m+1, DateSep, d, DateSep, y);
 #define L_G_OVER sprintf(s, "%s%s %d. %s%s", DayName[jul%7], L_ON, d, \
 			 MonthName[m], L_PARTIT);
-#define L_H_OVER sprintf(s, "%02d%c%02d", d, DATESEP, m+1);
-#define L_I_OVER sprintf(s, "%02d%c%02d", m+1, DATESEP, d);
+#define L_H_OVER sprintf(s, "%02d%c%02d", d, DateSep, m+1);
+#define L_I_OVER sprintf(s, "%02d%c%02d", m+1, DateSep, d);
 #define L_J_OVER sprintf(s, "%s%s %sn %d%s %d", DayName[jul%7], L_ON, \
 			 MonthName[m], d, plu, y);
 #define L_K_OVER sprintf(s, "%s%s %sn %d%s", DayName[jul%7], L_ON, \
 			 MonthName[m], d, plu);
-#define L_L_OVER sprintf(s, "%04d%c%02d%c%02d", y, DATESEP, m+1, DATESEP, d);
+#define L_L_OVER sprintf(s, "%04d%c%02d%c%02d", y, DateSep, m+1, DateSep, d);
 #define L_Q_OVER sprintf(s, "n");
 #define L_U_OVER sprintf(s, "%s%s %d%s %s%s %d", DayName[jul%7], L_ON, \
 			 d, plu, MonthName[m], L_PARTIT, y);
@@ -284,7 +282,7 @@ EXTERN char *ErrMsg[] =
     "AT-sanan per\xE4st\xE4 puuttuu aika",
     "UNTIL-sanaa k\xE4ytetty kahdesti",
     "Ep\xE4t\xE4ydellinen p\xE4iv\xE4ys",
-    "SCANFROM-sanaa k\xE4ytetty kahdesti",
+    "FROM/SCANFROM-sanaa k\xE4ytetty kahdesti",
     "Muuttuja",
     "Arvo",
     "*M\xC4\xC4RITTELEM\xC4T\xD6N*",
@@ -308,7 +306,8 @@ EXTERN char *ErrMsg[] =
     "Ei viestej\xE4.",
     "%d viesti(\xE4) t\xE4m\xE4n p\xE4iv\xE4n jonossa.\n",
     "Numero puuttuu",
-    "Virheellinen funktio WARN-lausekkeessa"
+    "Virheellinen funktio WARN-lausekkeessa",
+    "Can't convert between time zones"
 
 #elif IBMEXTENDED
     "Ok",
@@ -385,7 +384,7 @@ EXTERN char *ErrMsg[] =
     "AT-sanan per\x84st\x84 puuttuu aika",
     "UNTIL-sanaa k\x84ytetty kahdesti",
     "Ep\x84t\x84ydellinen p\x84iv\x84ys",
-    "SCANFROM-sanaa k\x84ytetty kahdesti",
+    "FROM/SCANFROM-sanaa k\x84ytetty kahdesti",
     "Muuttuja",
     "Arvo",
     "*M\x8E\x8ERITTELEM\x8ET\x99N*",
@@ -409,7 +408,8 @@ EXTERN char *ErrMsg[] =
     "Ei viestej\x84.",
     "%d viesti(\x84) t\x84m\x84n p\x84iv\x84n jonossa.\n",
     "Numero puuttuu"
-    "Virheellinen funktio WARN-lausekkeessa"
+    "Virheellinen funktio WARN-lausekkeessa",
+    "Can't convert between time zones"
 #else
     "Ok",
     "Puuttuva ']'",
@@ -485,7 +485,7 @@ EXTERN char *ErrMsg[] =
     "AT-sanan per{st{ puuttuu aika",
     "UNTIL-sanaa k{ytetty kahdesti",
     "Ep{t{ydellinen p{iv{ys",
-    "SCANFROM-sanaa k{ytetty kahdesti",
+    "FROM/SCANFROM-sanaa k{ytetty kahdesti",
     "Muuttuja",
     "Arvo",
     "*M[[RITTELEM[T\\N*",
@@ -509,7 +509,9 @@ EXTERN char *ErrMsg[] =
     "Ei viestej{.",
     "%d viesti({) t{m{n p{iv{n jonossa.\n",
     "Numero puuttuu",
-    "Virheellinen funktio WARN-lausekkeessa"
+    "Virheellinen funktio WARN-lausekkeessa",
+    "Can't convert between time zones"
+
 #endif
 };
 #endif /* MK_GLOBALS */
@@ -517,11 +519,7 @@ EXTERN char *ErrMsg[] =
 /* The following is only used in init.c */
 #ifdef L_IN_INIT
 #define L_USAGE_OVERRIDE 1
-#ifdef HAVE_PROTOS
-PUBLIC void Usage(void)
-#else
-void Usage()
-#endif /* HAVE_PROTOS */
+void Usage(void)
 {
     fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-1998 David F. Skoll\n", VERSION, L_LANGNAME);
     fprintf(ErrFp, "Copyright 1999-2000 Roaring Penguin Software Inc.\n");
