@@ -15,8 +15,6 @@
 /*                                                             */
 /***************************************************************/
 
-/* $Id: french.h,v 1.9 2000/02/18 03:53:50 dfs Exp $ */
-
 /* The very first define in a language support file must be L_LANGNAME: */
 #define L_LANGNAME "French"
 
@@ -221,7 +219,7 @@ EXTERN char *ErrMsg[] =
     "Heure attendue apr\350s AT",
     "Mot-cl\351 UNTIL utilis\351 deux fois",
     "Sp\351cification de date incompl\350te",
-    "Mot-cl\351 SCANFROM utilis\351 deux fois",
+    "Mot-cl\351 FROM/SCANFROM utilis\351 deux fois",
     "Variable",
     "Valeur",
     "*NON-DEFINI*",
@@ -245,7 +243,8 @@ EXTERN char *ErrMsg[] =
     "Pas de rappels.",
     "%d rappel(s) en file pour aujourd'hui.\n",
     "Nombre attendu",
-    "Fonction ill\351gale apr\350s WARN"
+    "Fonction ill\351gale apr\350s WARN",
+    "Can't convert between time zones"
 #else /* ISOLATIN1 */
     "Ok",
     "']' manquant",
@@ -321,7 +320,7 @@ EXTERN char *ErrMsg[] =
     "Heure attendue apres AT",
     "Mot-cle UNTIL utilise deux fois",
     "Specification de date incomplete",
-    "Mot-cle SCANFROM utilise deux fois",
+    "Mot-cle FROM/SCANFROM utilise deux fois",
     "Variable",
     "Valeur",
     "*NON-DEFINI*",
@@ -345,7 +344,8 @@ EXTERN char *ErrMsg[] =
     "Pas de rappels.",
     "%d rappel(s) en file pour aujourd'hui.\n",
     "Nombre attendu",
-    "Fonction illegale apres WARN"
+    "Fonction illegale apres WARN",
+    "Can't convert between time zones"
 #endif /* ISOLATIN1 */
 };
 #endif /* MK_GLOBALS */
@@ -353,11 +353,7 @@ EXTERN char *ErrMsg[] =
 /* The following is only used in init.c */
 #ifdef L_IN_INIT
 #define L_USAGE_OVERRIDE 1
-#ifdef HAVE_PROTOS
-PUBLIC void Usage(void)
-#else
-void Usage()
-#endif /* HAVE_PROTOS */
+void Usage(void)
 {
     fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-1998 David F. Skoll\n", VERSION, L_LANGNAME);
     fprintf(ErrFp, "Copyright 1999-2000 Roaring Penguin Software Inc.\n");
