@@ -10,8 +10,6 @@
 /*                                                             */
 /***************************************************************/
 
-/* $Id: protos.h,v 1.10 2005/09/30 03:29:32 dfs Exp $ */
-
 /* Define a string assignment macro - be careful!!! */
 #define STRSET(x, str) { if (x) free(x); (x) = StrDup(str); }
 
@@ -24,6 +22,7 @@ int CallUserFunc (char *name, int nargs);
 int DoFset (ParsePtr p);
 void ProduceCalendar (void);
 char  *SimpleTime (int tim);
+char  *CalendarTime (int tim, int duration);
 int DoRem (ParsePtr p);
 int DoFlush (ParsePtr p);
 void DoExit (ParsePtr p);
@@ -134,4 +133,4 @@ void HuntPhase (int startdate, int starttim, int phas, int *date, int *time);
 int CompareRems (int dat1, int tim1, int prio1, int dat2, int tim2, int prio2, int bydate, int bytime, int byprio);
 void SigIntHandler (int d);
 void GotSigInt (void);
-
+void SynthesizeTag(char *);
