@@ -12,7 +12,6 @@
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: token.c,v 1.13 2005/11/20 01:26:59 dfs Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -35,8 +34,6 @@ while (isdigit(*(string))) { \
     string++; \
 }
 
-#define UPPER(c) (islower(c) ? toupper(c) : c)
-
 /* The big array holding all recognized (literal) tokens in reminder file.
    Keep this array sorted, or software will not work. */
 Token TokArray[] = {
@@ -61,6 +58,7 @@ Token TokArray[] = {
     { "february",	3,	T_Month,	1 },
     { "flush",		5,	T_Flush,	0 },
     { "friday",		3,	T_WkDay,	4 },
+    { "from",		4,	T_Scanfrom,	FROM_TYPE },
     { "fset",		4,	T_Fset,		0 },
     { "if",		2,	T_If,		0 },
     { "iftrig",		6,	T_IfTrig,	0 },
@@ -87,7 +85,7 @@ Token TokArray[] = {
     { "run",		3,	T_RemType,	RUN_TYPE },
     { "satisfy",	7,	T_RemType,      SAT_TYPE },
     { "saturday",	3,	T_WkDay,	5 },
-    { "scanfrom",	4,	T_Scanfrom,	0 },
+    { "scanfrom",	4,	T_Scanfrom,	SCANFROM_TYPE },
     { "sched",		5,	T_Sched,	0 },
     { "september",	3,	T_Month,	8 },
     { "set",		3,	T_Set,		0 },
@@ -278,7 +276,7 @@ void FindNumericToken(const char *s, Token *t)
 	    return;
 	}
 	/* If we hit a colon or a period, we've probably got a time hr:min */
-	if (*s == ':' || *s == '.' || *s == TIMESEP) {
+	if (*s == ':' || *s == '.' || *s == TimeSep) {
 	    s++;
 	    hour = t->val;
 	    PARSENUM(min, s);
@@ -342,7 +340,7 @@ static int TokStrCmp(const Token *t, const char *s)
     register int r;
     char *tk = t->name;
     while(*tk && *s && !(*s == ',' && *(s+1) == 0)) {
-	r = UPPER(*tk) - UPPER(*s);
+	r = *tk - tolower(*s);
 	tk++;
 	s++;
 	if (r) return r;
