@@ -11,8 +11,6 @@
 /***************************************************************/
 
 #include "config.h"
-static char const RCSID[] = "$Id: trigger.c,v 1.6 2005/09/30 03:29:32 dfs Exp $";
-
 #include <stdio.h>
 
 #include <stdlib.h>
@@ -352,11 +350,12 @@ int ComputeTrigger(int today, Trigger *trig, int *err)
 {
     int nattempts = 0,
 	start = today,
-	nextstart,
+	nextstart = 0,
 	y, m, d,
 	result;
 
     LastTrigValid = 0;
+
 /* Assume everything works */
     *err = OK;
 
