@@ -10,8 +10,6 @@
 /*                                                             */
 /***************************************************************/
 
-/* $Id: types.h,v 1.7 2005/11/20 01:26:59 dfs Exp $ */
-
 #include <limits.h>
 #include "dynbuf.h"
 
@@ -169,8 +167,10 @@ typedef struct {
 #define IF_ELSE_MASK 2
 
 /* Flags for the DoSubst function */
-#define NORMAL_MODE 0
-#define CAL_MODE 1
+#define NORMAL_MODE  0
+#define CAL_MODE     1
+#define ADVANCE_MODE 2
+
 #define QUOTE_MARKER 1 /* Unlikely character to appear in reminder */
 
 /* Flags for disabling run */
@@ -187,3 +187,7 @@ typedef struct {
 #define SORT_NONE    0
 #define SORT_ASCEND  1
 #define SORT_DESCEND 2
+
+/* Flags for FROM / SCANFROM */
+#define SCANFROM_TYPE 0
+#define FROM_TYPE     1
